#
# dashboard_helpers.py - Various utilities for packager dashboard
#
# Copyright 2020, Red Hat, Inc
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# Authors:
#   Frantisek Zatloukal <fzatlouk@redhat.com>
#   Josef Skladanka <jskladan@redhat.com>

import json
import yaml
import requests
import datetime
import fedfind.helpers
import re

from urllib3.util.retry import Retry
from requests.adapters import HTTPAdapter
from requests.exceptions import ConnectionError, RetryError
from json import JSONDecodeError

from oraculum import app, CACHE, db
from oraculum.models.dashboard_users import DashboardUserData


def get_json(json_url, attempt_retry=True, log_errors=True, ignore_json_errors=False):
    """
    Returns json data from provided url
    """
    session = requests.Session()

    if attempt_retry:
        retries = Retry(total=3,
                        backoff_factor=0.1,
                        status_forcelist=[429, 500, 502, 503, 504])
        session.mount('https://', HTTPAdapter(max_retries=retries))
    else:
        session.mount('https://', HTTPAdapter())
    try:
        resp = session.get(json_url, timeout=60)
    except (ConnectionError, RetryError):
        # Return none if retrying failed too
        if log_errors:
            app.logger.error("Request to %s failed." % json_url)
        return None
    if not attempt_retry:
        try:
            return json.loads(resp.text)
        except JSONDecodeError:
            if ignore_json_errors:
                return resp.text
            if log_errors:
                app.logger.error("Request to %s failed because resource returned invalid data." % json_url)
            return None
    try:
        return json.loads(resp.text)
    except JSONDecodeError:
        if ignore_json_errors:
            return resp.text
        get_json(json_url, attempt_retry=False, log_errors=log_errors, ignore_json_errors=ignore_json_errors)

def branched_frozen():
    """
    Returns true if current branched is frozen
    """
    result = get_json("https://pagure.io/fedora-infra/ansible/raw/master/f/vars/all/Frozen.yaml", ignore_json_errors=True)
    try:
        return yaml.safe_load(result)["Frozen"]
    except (KeyError, AttributeError):
        app.logger.error("Failed to detect freeze status for branched...")
        return None

def get_fedora_releases():
    """
    Returns dict with keys containing active Fedora releases as described by keys.
    "values" key contains list of active Fedora release numbers
    """
    releases = {"oldstable": (max(fedfind.helpers.get_current_stables()) - 1),
                "stable": max(fedfind.helpers.get_current_stables()),
                "branched": fedfind.helpers.get_current_release(branched=True),
                "rawhide": (fedfind.helpers.get_current_release(branched=True) + 1)}

    releases["values"] = sorted(set(releases.values()))
    if releases["stable"] == releases["branched"]:
        releases["branched"] = None
        releases["branched_frozen"] = None
    else:
        releases["branched_frozen"] = branched_frozen()
    return releases


def update_user_access_time(user):
    """
    Updates user last_accessed with current timestamp
    """
    row = DashboardUserData.query.filter_by(username=user).first()
    if not row:
        row = DashboardUserData(user, datetime.datetime.utcnow())
        db.session.add(row)
    else:
        row.last_accessed = datetime.datetime.utcnow()
    db.session.commit()


def name_in_nevra(name, nevra):
    """
    Checks if name/nevra matches
    """
    #!!! - hawkey.split_nevra leaks memory, so we have to do this shit instead... FUN!
    #if name == hawkey.split_nevra(nevra).name:
    if name == nevra.rsplit('-', 2)[0]:
        return True
    else:
        return False


def release_from_number(release_num):
    """
    Returns release string (Fedora XX) from number (or "rawhide" string)
    Returns "Fedora Rawhide" if release number is equal to Rawhide number or "rawhide"
    """
    releases = CACHE.get("fedora_releases")

    if str(release_num).lower() == "rawhide" or int(release_num) == releases["rawhide"]:
        return "Fedora Rawhide"
    if int(release_num) in app.config["EPEL_RELEASES"] or int(release_num) <= 6:
        return "EPEL %s" % release_num
    return "Fedora %s" % release_num


def release_from_nvr(nvr):
    """
    Returns Fedora XX or EPEL X from nvr
    """
    split = nvr.split(".")[-1]
    if len(split) <= 1:
        return "Unknown"
    if "fedora-obsolete-packages" in nvr:
        # fedora-obsolete-packages does not contain distag, workaround it
        # eg. "fedora-obsolete-packages-32-51", release is "Fedora 32"
        return release_from_dist(nvr.split("-")[3])
    return release_from_dist(split)


def release_from_dist(dist):
    """
    Returns Fedora XX or EPEL X from dist (fcXX/elX or fXX/epelX)
    """
    releases = CACHE.get("fedora_releases")
    dist = dist.replace("~bootstrap", "")
    if "el" in dist or "epel" in dist.lower():
        return "EPEL %s" % dist[-1:]

    # Let's check if it's Fedora branch at all
    re_rule = re.compile(r'^f.\d+$')

    if not re_rule.match(dist.lower()):
        return "Fedora Unknown"

    if int(dist[-2:]) == releases["rawhide"]:
        return "Fedora Rawhide"
    return "Fedora %s" % dist[-2:]


def release_is_active(release_string):
    """
    Gets release string (Fedora XX/EPEL X), returns False if the release is EOL, return True otherwise
    """
    releases = CACHE.get("fedora_releases")
    if "Rawhide" in release_string:
        return True
    release_string = int(release_string.replace("Fedora ", "").replace("EPEL ", "")) # Get just the number
    if release_string in releases["values"] or release_string in app.config["EPEL_RELEASES"]:
        return True
    return False
