import pytest
import mock

from oraculum.utils import bugzilla
from oraculum import app, CACHE, BUGZILLA


class dotdict(dict):
    # https://stackoverflow.com/a/23689767
    """dot.notation access to dictionary attributes"""
    __getattr__ = dict.get
    __setattr__ = dict.__setitem__
    __delattr__ = dict.__delitem__

def bz_advanced_releases_stub(what):
    if "fedora_releases" in what:
        return {
            "oldstable": 96,
            "stable": 97,
            "branched": 98,
            "rawhide": 99,
            "values": [96, 97, 98, 99]
        }
    if "ftbfs_trackers" in what:
        return [1,2]
    if "fti_trackers" in what:
        return [3,4]
    if "blocker_trackers" in what:
        return [5,6]

def bz_query(query):
    dct = {
        "summary": "Some bug description",
        "creator": "Happy Tester",
        "id": 1,
        "blocks": [],
        "severity": "unspecified",
        "priority": "unspecified",
        "status": "NEW",
        "last_change_time": "2020-10-01T12:21:44Z",
        "creation_time": "2020-01-24T22:08:29Z",
        "version": "97",
        "keywords": [],
        "whiteboard": "",
        "comments": [None, None]
    }

    if query["pkg_name"] == "package_b":
        dct["blocks"] = [5]
        dct["whiteboard"] = "AcceptedBlocker"
        dct["version"] = "rawhide"

    if query["pkg_name"] == "package_c":
        dct["blocks"] = [6]
        dct["whiteboard"] = "ProposedBlocker"
        dct["version"] = "98"

    if query["pkg_name"] == "package_d":
        dct["blocks"] = [1]

    if query["pkg_name"] == "package_e":
        dct["blocks"] = [3]
        dct["version"] = "96"

    if query["pkg_name"] == "package_f":
        dct["summary"] = "[fedora-all]Some bug description"
        dct["priority"] = "high"
        dct["severity"] = "urgent"

    return [dotdict(dct)]


class TestBugzilla(object):


    stub_fedora_releases = {
        "oldstable": 96,
        "stable": 97,
        "branched": 98,
        "rawhide": 99,
        "values": [96, 97, 98, 99]
    }

    def test_trackers(self, monkeypatch):

        expected_result = [961111, 971111, 981111, 991111]

        monkeypatch.setattr(CACHE, 'get', mock.MagicMock(return_value=self.stub_fedora_releases))
        monkeypatch.setattr(BUGZILLA, 'getbug', lambda req: dotdict({"id": int(req[1:3] + "1111")}))

        # Blockers aren't fetched for anything else than Rawhide and Branched
        assert bugzilla.get_blocker_trackers() == [981111, 981111, 991111, 991111]

        assert bugzilla.get_ftbfs_trackers() == expected_result
        assert bugzilla.get_fti_trackers() == expected_result

    def test_package_bugs(self, monkeypatch):

        expected_result = [{
            'bug_id': 1,
            'comments': 1,
            'keywords': [],
            'modified': '2020-10-01 12:21:44',
            'priority': 'unspecified',
            'priority_severity': 'unspecified',
            'release': 'Fedora 97',
            'reported': '2020-01-24 22:08:29',
            'severity': 'unspecified',
            'status': 'NEW',
            'title': 'Some bug description',
            'url': 'https://bugzilla.redhat.com/1'
        }]

        expected_result_b = [{**expected_result[0]}]
        expected_result_c = [{**expected_result[0]}]
        expected_result_d = [{**expected_result[0]}]
        expected_result_e = [{**expected_result[0]}]
        expected_result_f = [{**expected_result[0]}]

        expected_result_b[0]["keywords"] = ["AcceptedBlocker"]
        expected_result_b[0]["release"]  = "Fedora Rawhide"

        expected_result_c[0]["keywords"] = ["ProposedBlocker"]
        expected_result_c[0]["release"]  = "Fedora 98"

        expected_result_d[0]["keywords"] = ["FTBFS"]

        expected_result_e[0]["keywords"] = ["FTI"]
        expected_result_e[0]["release"]  = "Fedora 96"

        expected_result_f[0]["priority"] = "high"
        expected_result_f[0]["severity"] = "urgent"
        expected_result_f[0]["release"]  = "Fedora"
        expected_result_f[0]["title"]    = "[fedora-all]Some bug description"
        expected_result_f[0]["priority_severity"] = "urgent"

        monkeypatch.setattr(CACHE, 'get', bz_advanced_releases_stub)
        monkeypatch.setattr(BUGZILLA, 'url_to_query', lambda url: {"pkg_name": url.split("component=")[1]})
        monkeypatch.setattr(BUGZILLA, 'query', bz_query)

        assert bugzilla.get_package_bugs("package_a") == expected_result # Normal F97 bug
        assert bugzilla.get_package_bugs("package_b") == expected_result_b # F99 Accepted Blocker
        assert bugzilla.get_package_bugs("package_c") == expected_result_c # F98 Proposed Blocker
        assert bugzilla.get_package_bugs("package_d") == expected_result_d # F97 FTBFS
        assert bugzilla.get_package_bugs("package_e") == expected_result_e # F96 FTI
        assert bugzilla.get_package_bugs("package_f") == expected_result_f # Fedora CVE, High priotiy, Urgent severity
