import pytest
import mock
import fedfind.helpers

from oraculum.utils import dashboard_helpers
from oraculum import app, CACHE


class TestDashboardHelpers(object):


    stub_fedora_releases = {
        "oldstable": 96,
        "stable": 97,
        "branched": 98,
        "rawhide": 99,
        "values": [96, 97, 98, 99]
    }

    def test_release_is_active(self, monkeypatch):

        # Force EPEL Releases for tests
        # This is usually defined in settings.py by real values, so we can't put it in config.py TEST block
        app.config["EPEL_RELEASES"] = [1, 2]
        monkeypatch.setattr(CACHE, 'get', mock.MagicMock(return_value=self.stub_fedora_releases))

        assert dashboard_helpers.release_is_active("Fedora Rawhide")
        assert dashboard_helpers.release_is_active("Fedora 97")
        assert dashboard_helpers.release_is_active("EPEL 1")
        assert not dashboard_helpers.release_is_active("Fedora 91")
        assert not dashboard_helpers.release_is_active("EPEL 3")

    def test_release_from_dist(self, monkeypatch):

        monkeypatch.setattr(CACHE, 'get', mock.MagicMock(return_value=self.stub_fedora_releases))
        assert dashboard_helpers.release_from_dist("fc99") == "Fedora Rawhide"
        assert dashboard_helpers.release_from_dist("fc98") == "Fedora 98"
        assert dashboard_helpers.release_from_dist("f98") == "Fedora 98"
        assert dashboard_helpers.release_from_dist("fc90") == "Fedora 90"
        assert dashboard_helpers.release_from_dist("fc99s") == "Fedora Unknown"

        assert dashboard_helpers.release_from_dist("el1") == "EPEL 1"
        assert dashboard_helpers.release_from_dist("epel1") == "EPEL 1"
        assert dashboard_helpers.release_from_dist("el1~bootstrap") == "EPEL 1"

    def test_release_from_nevra(self, monkeypatch):

        app.config["EPEL_RELEASES"] = [1, 2]
        monkeypatch.setattr(CACHE, 'get', mock.MagicMock(return_value=self.stub_fedora_releases))
        assert dashboard_helpers.release_from_nvr("package-1.2.1-1.fc99") == "Fedora Rawhide"
        assert dashboard_helpers.release_from_nvr("package-1:1.2.1-1.fc98") == "Fedora 98"
        assert dashboard_helpers.release_from_nvr("package-2.2.1-1.fc16") == "Fedora 16"
        assert dashboard_helpers.release_from_nvr("package-0.1.1-1.el1") == "EPEL 1"
        assert dashboard_helpers.release_from_nvr("package-0.1.1-1.el2~bootstrap") == "EPEL 2"

    def test_release_from_number(self, monkeypatch):

        app.config["EPEL_RELEASES"] = [1, 2]
        monkeypatch.setattr(CACHE, 'get', mock.MagicMock(return_value=self.stub_fedora_releases))
        assert dashboard_helpers.release_from_number("99") == "Fedora Rawhide"
        assert dashboard_helpers.release_from_number("rawhide") == "Fedora Rawhide"
        assert dashboard_helpers.release_from_number("98") == "Fedora 98"
        assert dashboard_helpers.release_from_number("90") == "Fedora 90"
        assert dashboard_helpers.release_from_number("1")  == "EPEL 1"

    def test_name_in_nevra(self, monkeypatch):

        assert dashboard_helpers.name_in_nevra("package", "package-1:1.2.1-1.fc98")
        assert dashboard_helpers.name_in_nevra("package", "package-1.2.1-1.fc99")
        assert dashboard_helpers.name_in_nevra("package-a", "package-a-1:1.2.1-1.el1.x86_64")
        assert not dashboard_helpers.name_in_nevra("Package", "package-1:1.2.1-1.el2")

    def test_get_fedora_releases(self, monkeypatch):

        expected_result = {
            'branched': 98,
            'branched_frozen': True,
            'oldstable': 96,
            'rawhide': 99,
            'stable': 97,
            'values': [96, 97, 98, 99]
        }

        monkeypatch.setattr(fedfind.helpers, 'get_current_stables', mock.MagicMock(return_value=[95,96,97]))
        monkeypatch.setattr(fedfind.helpers, 'get_current_release', mock.MagicMock(return_value=98))
        monkeypatch.setattr(dashboard_helpers, 'branched_frozen', mock.MagicMock(return_value=True))
        assert dashboard_helpers.get_fedora_releases() == expected_result

    def test_branched_frozen(self, monkeypatch):

        yaml_response = "Frozen: True\n"

        monkeypatch.setattr(dashboard_helpers, 'get_json', mock.MagicMock(return_value=yaml_response))
        assert dashboard_helpers.branched_frozen()
