import pytest
import mock

from oraculum.utils import pagure
from oraculum import app, CACHE


def stub_get_json_for_pagure_groups(url):

    stub_groups = {
        "groups": [
            "group_not_to_be_included",
            "included-cause-sig",
            "sig-but-not-really",
            "sssd-maintainers"
        ]
    }

    stub_group = {
        "members": ["jdoe", "mr_packager"],
        "projects": [
            {
                "name": "glorious_package"
            },
            {
                "name": "some_other_package"
            }
        ]
    }

    if "projects=1&acl=commit" in url:
        return stub_group
    return stub_groups


def stub_get_json_for_pagure_versions(url, *args):

    # Define data we're going to return
    stub_pagure_versions_mdapi = {
        "version": "80.0.1",
        "release": "1.fc98"
    }

    stub_pagure_versions_complete = {
        "updates": {
            "EL99": {"stable": None, "testing": None},
            "EL100": {"stable": None, "testing": None},
            "F96": {"stable": "package-80.0.1-1.fc96", "testing": "package-80.0.1-2.fc96"},
            "F97": {"stable": "package-80.0.1-1.fc97", "testing": "package-80.0.1-2.fc97"},
            "F98": {"stable": "package-80.0.1-1.fc98", "testing": "package-80.0.1-1.fc98"},
            "F99": {"stable": "package-80.0.1-1.fc98", "testing": "package-80.0.1-1.fc98"},
        }
    }

    stub_pagure_versions_incomplete = {
        "updates": {
            "F96": {"stable": "package-80.0.1-1.fc96", "testing": "package-80.0.1-2.fc96"},
            "F97": {"stable": "package-80.0.1-1.fc97", "testing": "package-80.0.1-2.fc97"},
            "F98": {"stable": None, "testing": "package-80.0.1-1.fc98"},
            "F99": {"stable": None, "testing": "package-80.0.1-1.fc98"},
        }
    }

    if "bodhi_updates" in url:
        if "complete" in url:
            return stub_pagure_versions_complete
        else:
            return stub_pagure_versions_incomplete
    return stub_pagure_versions_mdapi


class TestPagure(object):


    def test_package_prs(self, monkeypatch):

        stub_pagure_prs_data = {
            "requests": [
                {
                    "id": 0,
                    "status": "Open",
                    "title": "Open PR Title",
                    "date_created": "1598478958",
                    "last_updated": "1568380289",
                    "branch": "master",
                    "user": {
                        "name": "John Doe"
                    },
                    "comments": [{}, {}]
                },
                {
                    "id": 1,
                    "status": "Closed",
                    "title": "Open PR Title",
                    "date_created": "1598478958",
                    "last_updated": "1568380289",
                    "branch": "master",
                    "user": {
                        "name": "John Doe"
                    },
                    "comments": [{}, {}]
                }
            ]
        }

        monkeypatch.setattr(pagure, 'get_json', mock.MagicMock(return_value=stub_pagure_prs_data))
        monkeypatch.setattr(pagure, 'get_pr_ci_result', mock.MagicMock(return_value=None))
        result = pagure.get_package_prs('testPackage')
        assert len(result) == 1
        assert result[0]["author"] == "John Doe"
        assert result[0]["release"] == "Fedora Rawhide"


    def test_package_pr_cis(self, monkeypatch):

        stub_pagure_prs_cis_data = {
            "flags": [
                {
                    "username": "simple-koji-ci",
                    "status": "failure"
                },
                {
                    "username": "Zuul",
                    "status": "success"
                },
                {
                    "username": "Zuul",
                    "status": "failure"
                }
            ]
        }

        monkeypatch.setattr(pagure, 'get_json', mock.MagicMock(return_value=stub_pagure_prs_cis_data))
        result = pagure.get_pr_ci_result('testPackage', 0)
        assert len(result) == 2
        assert result["Zuul"] == "success"
        assert result["simple-koji-ci"] == "failure"


    def test_pagure_groups(self, monkeypatch):

        stub_pkg_owners = {
            "rpms": {
                "directly_jdoes_package": ["jdoe", "somebody"],
                "no_package": ["nobody"]
            }
        }

        expected_groups_result = {
            "included-cause-sig": {
                "users": ["jdoe", "mr_packager"],
                "packages": ["glorious_package", "some_other_package"]
            },
            "sssd-maintainers": {
                "users": ["jdoe", "mr_packager"],
                "packages": ["glorious_package", "some_other_package"]
            }
        }

        expected_user_group_result = {
            "included-cause-sig": ["glorious_package", "some_other_package"],
            "sssd-maintainers": ["glorious_package", "some_other_package"]
        }

        monkeypatch.setattr(pagure, 'get_json', stub_get_json_for_pagure_groups)
        result_groups = pagure.get_pagure_groups()
        assert result_groups == expected_groups_result

        # let's test the rest of pagure groups functions
        result_grp_pkgs = pagure.get_user_group_packages("jdoe", result_groups)
        assert set(["glorious_package", "some_other_package"]) == result_grp_pkgs[1]
        assert result_grp_pkgs[0] == expected_user_group_result

        result_complete = pagure.get_packages("jdoe", stub_pkg_owners, result_groups)
        assert ["directly_jdoes_package"] == result_complete["primary"]

        assert "glorious_package" in result_complete["combined"]
        assert "some_other_package" in result_complete["combined"]
        assert "directly_jdoes_package" in result_complete["combined"]

        assert expected_user_group_result == result_complete["group"]


    def test_pagure_versions(self, monkeypatch):
        pagure_versions_result = {
            "EPEL 1": {"stable": None, "testing": None},
            "EPEL 2": {"stable": None, "testing": None},
            "Fedora 96": {"stable": "package-80.0.1-1.fc96", "testing": "package-80.0.1-2.fc96"},
            "Fedora 97": {"stable": "package-80.0.1-1.fc97", "testing": "package-80.0.1-2.fc97"},
            "Fedora 98": {"stable": "package-80.0.1-1.fc98", "testing": "package-80.0.1-1.fc98"},
            "Fedora Rawhide": {"stable": "package-80.0.1-1.fc98", "testing": "package-80.0.1-1.fc98"}
        }

        stub_fedora_releases = {
            "oldstable": 96,
            "stable": 97,
            "branched": 98,
            "rawhide": 99,
            "values": [96, 97, 98, 99]
        }

        # Force EPEL Releases for tests
        # This is usually defined in settings.py by real values, so we can't put it in config.py TEST block
        app.config["EPEL_RELEASES"] = [1, 2]

        monkeypatch.setattr(CACHE, 'get', mock.MagicMock(return_value=stub_fedora_releases))
        monkeypatch.setattr(pagure, 'get_json', stub_get_json_for_pagure_versions)

        assert pagure.get_package_versions("completePackage") == pagure_versions_result
        assert pagure.get_package_versions("package") == pagure_versions_result
